local function clamp(x, min, max)
	return math.max(math.min(x, max), min)
end

function love.load()
	if not love.graphics.isSupported("pixeleffect") then
		error("pixel effects (shaders) not supported on this system!")
		return
	end
	
	if not love.graphics.isSupported("canvas") then
		error("canvases not supported on this system!")
		return
	end
	
	if not love.graphics.isSupported("npot") then
		error("non power of two sized canvases not supported on this system!")
		return
	end
	
	time = 0
	maxtime = 8
	
	logo = love.graphics.newImage("logo.png")
	
	screencanvas = love.graphics.newCanvas()
	
	local blurscale = 1
	blurcanvases = {}
	for i=1, 2 do
		blurcanvases[i] = love.graphics.newCanvas(love.graphics.getWidth()/(1/blurscale), love.graphics.getHeight()/(1/blurscale))
	end
	
	shaders = loadShaders()
	
	shaders.blurhorizontal:send("canvas_w", blurcanvases[1]:getWidth())
	shaders.blurvertical:send("canvas_h", blurcanvases[1]:getHeight())
	
	blurIntensityX = 12
	blurIntensityY = 1
	setBlurIntensity(blurIntensityX, blurIntensityY)
	
	setSeparationAmount("red", 0, 0)
	setSeparationAmount("green", 0, 0)
	setSeparationAmount("blue", 0, 0)
end

function drawScene()
	love.graphics.setBlendMode("alpha")
	love.graphics.setColor(255, 255, 255, 255 * clamp(time/(maxtime/3), 0, 1))
	love.graphics.draw(logo, love.graphics.getWidth()/2, love.graphics.getHeight()/2, 0, 1, 1, logo:getWidth()/2, logo:getHeight()/2)
end

function setSeparationAmount(color, x, y)
	shaders.colorseparator:send(color.."offset", {x, y})
end

function setBlurIntensity(x, y)
	shaders.blurhorizontal:send("intensity", x)
	shaders.blurvertical:send("intensity", y)
end

function drawBlurredStart()
	love.graphics.setColor(255, 255, 255)
	for i,v in ipairs(blurcanvases) do v:clear() end
	love.graphics.setCanvas(blurcanvases[1])
	love.graphics.push()
	love.graphics.scale(blurcanvases[1]:getWidth()/love.graphics.getWidth(), blurcanvases[1]:getHeight()/love.graphics.getHeight())
end

function drawBlurredEnd()
	love.graphics.setColor(255, 255, 255)
	
	love.graphics.pop()
	
	love.graphics.setBlendMode("premultiplied")
	
	love.graphics.setCanvas(blurcanvases[2])
	love.graphics.setPixelEffect(shaders.blurhorizontal)
	love.graphics.draw(blurcanvases[1], 0, 0)
	
	love.graphics.setCanvas(screencanvas)
	love.graphics.setPixelEffect(shaders.blurvertical)
	love.graphics.draw(blurcanvases[2], 0, 0, 0, love.graphics.getWidth()/blurcanvases[2]:getWidth(), love.graphics.getHeight()/blurcanvases[2]:getHeight())
end

function loadShaders()
	local shaders = {}
	local folder = "shaders"
	
	for i,v in ipairs(love.filesystem.enumerate(folder)) do
		local filename = folder.."/"..v
		local name, ext = v:match("^(.+)%.([^%.]-)$")
		if love.filesystem.exists(filename) and love.filesystem.isFile(filename) and ext == "frag" then
			shaders[name] = love.graphics.newPixelEffect(filename)
			-- print(string.format("loaded shader %q", name))
		end
	end
	
	return shaders
end

function love.update(dt)
	time = time + dt
	
	blurIntensityX = math.max(blurIntensityX - dt * 2, 0)
	blurIntensityY = clamp(maxtime - time - 2, 0, 1)
	
	setBlurIntensity(blurIntensityX, blurIntensityY)
	
	local offsets = {
		red = {-2 - clamp(maxtime - time - 2, 0, 2), 0},
		green = {0, clamp((3 - time) * 2, 0, 6)},
		blue = {2 + clamp(maxtime - time - 2, 0, 2), 0},
	}
	
	for k,v in pairs(offsets) do
		setSeparationAmount(k, v[1], v[2])
	end
end

function love.draw()
	love.graphics.setPixelEffect()
	screencanvas:clear()
	
	drawBlurredStart()
	
	drawScene()
	
	drawBlurredEnd()
	
	love.graphics.setPixelEffect(shaders.colorseparator)
	love.graphics.setCanvas()
	love.graphics.setBlendMode("premultiplied")
	love.graphics.draw(screencanvas, 0, 0)
end

